/*
 * TeamStats by Mats Bovin (mats@mbovin.com)
 * v1.3.0 September 2002
 * Copyright (c) 2002 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <PalmOS.h>
#include "TeamStats.h"
#include "rcp.h"
#include "League.h"
#include "Funcs.h"
#include "Text.h"

#define createLeagueErr 1


// Globals used by "Edit league"
MemHandle GLeagueNameHandle;
Char*     GLineNames[MAX_TEAMS + 1];
// Globals used by "Edit teams"
Char*     GDialogTeamNames[MAX_TEAMS];
MemHandle GTeamNameHandle;
MemHandle GTeamPointsHandle;
// Globals used by "Edit fixtures"
Int8  GRound;
Int16 GYear;
Int16 GMonth;
Int16 GDay;
Char* GMatchStrings[MAX_TEAMS];


//
// Create league
//

// Tries to create a new league from information in the "Create league" form.
// Returns any error that ocurred when the db was created or 'createLeagueErr'
// if the form wasn't correctly filled in.
Err SubmitCreateLeagueForm() {
  MemHandle textHandle;
  Char*     namePtr;
  League    league;
  Err       err = 0;
  UInt16    pointGroupIndex;

  // get winpoints and teamcount from form
  pointGroupIndex  = FrmGetControlGroupSelection(FrmGetActiveForm(), groupID_winPoints);
  league.winPoints = (FrmGetObjectId(FrmGetActiveForm(), pointGroupIndex) == btnID_3point) ? 3 : 2;
  league.teamCount = UIGetFieldIntValue(fieldID_teamCount);
  // get handle for name field
  textHandle = FldGetTextHandle((FieldPtr)UIGetObjectPtr(fieldID_leaguename));

  if(textHandle != NULL) {
    namePtr = MemHandleLock(textHandle);
    // if the entered info is ok -> go ahead and create the db
    if((league.teamCount > 1) && (league.teamCount <= MAX_TEAMS) && (StrLen(namePtr) > 0)) {
      err = CreateDatabase(namePtr, &league);
    }
    else {
      err = createLeagueErr;
    }

    MemHandleUnlock(textHandle);
  }
  else {
    err = createLeagueErr;
  }

  return err;
}


// Eventhandler for "Create league" form.
Boolean CreateLeagueFormEvtHandler(EventPtr event) {
  Boolean handled = false;
  Err     err;

  switch (event->eType) {

 // form is opened
  case frmOpenEvent:
    FrmSetControlGroupSelection(FrmGetActiveForm(), groupID_winPoints, btnID_3point);
    FrmDrawForm(FrmGetActiveForm());
    handled = true;
    break;

 // a button was pressed
  case ctlSelectEvent:
    switch(event->data.ctlSelect.controlID) {

      // OK -> try and create the new league
      case btnID_OK:
        err = SubmitCreateLeagueForm();
        switch(err) {
          case 0: // no error
            FrmReturnToForm(0);
            ShowLeague(GState.leagueCount-1, MODE_TABLE, true);
            break;
          case createLeagueErr:
            FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_LEAGUE_NAME_NO_OF_TEAMS, NULL, NULL);
            break;
          case dmErrInvalidDatabaseName:
          case dmErrAlreadyExists:
            FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_LEAGUE_INVALID_NAME, NULL, NULL);
            break;
          default: // all other errors (not handled at the moment)
            FrmReturnToForm(0);
        }
        handled = true;
        break;

      // Cancel -> just return to previous form
      case btnID_Cancel:
        FrmReturnToForm(0);
        handled = true;
        break;
    }
    break;
  }

  return handled;
}


//
// Edit league
//

// Saves the new league info from the "Edit league" form.
Err SubmitEditLeagueForm() {
  Int16     i;
  Err       err = 0;
  MemHandle textHandle;
  Char*     namePtr;
  ListPtr   lineList;
  UInt16    pointGroupIndex;
  League    league = LeagueGetCurrent();

  // get winpoints from form
  pointGroupIndex  = FrmGetControlGroupSelection(FrmGetActiveForm(), groupID_winPoints);
  league.winPoints = (FrmGetObjectId(FrmGetActiveForm(), pointGroupIndex) == btnID_3point) ? 3 : 2;

  // get lines from form
  for (i = 0; i < 4; i++) {
    lineList = (ListPtr)UIGetObjectPtr(popListID_line0 + i);
    league.line[i] = LstGetSelection(lineList) - 1;
  }

  // check that a name was entered and save the league
  namePtr = MemHandleLock(GLeagueNameHandle);
  if(StrLen(namePtr) > 0) {
    err = EditCurrentDatabase(namePtr, &league);
  }
  else {
    err = createLeagueErr;
  }

  MemHandleUnlock(GLeagueNameHandle);

  return err;
}


// Initializes "Edit league" form.
void InitEditLeagueForm() {
  Int16   i;
  UInt16  btnID;
  ListPtr lineList;
  Char*   namePtr;
  League  league = LeagueGetCurrent();

  // select the correct radio button
  btnID = (league.winPoints == 3) ? btnID_3point : btnID_2point;
  FrmSetControlGroupSelection(FrmGetActiveForm(), groupID_winPoints, btnID);
  // create and set text handle for the league name field
  GLeagueNameHandle = MemHandleNew(LEAGUENAME_SIZE);
  namePtr = (Char*)MemHandleLock(GLeagueNameHandle);
  StrCopy(namePtr, GLeagueNames[GState.currentLeagueIndex]);
  MemHandleUnlock(GLeagueNameHandle);
  FldSetTextHandle((FieldPtr)UIGetObjectPtr(fieldID_leaguename), GLeagueNameHandle);

  // allocate memroy and initialize the data for the line drop-downs
  GLineNames[0] = (Char*)MemPtrNew(3);
  StrCopy(GLineNames[0], "-");
  for(i = 0; i < league.teamCount; i++) {
    GLineNames[i+1] = (Char*)MemPtrNew(3);
    StrIToA(GLineNames[(i+1)], i+1);
  }
  // give the pointer array to the list-controls
  for (i = 0; i < 4; i++) {
    lineList = (ListPtr)UIGetObjectPtr(popListID_line0 + i);
    LstSetListChoices(lineList, GLineNames, league.teamCount + 1);
    UISelectDropDown(popListID_line0 + i, popTrigID_line0 + i, league.line[i] + 1);
  }

}


// Frees resources allocated for this form.
void FreeEditLeagueForm() {
  Int16  i;
  League league = LeagueGetCurrent();

  // free memory for line drop-downs
  for (i = 0; i < league.teamCount+1; i++) {
    MemPtrFree(GLineNames[i]);
  }
  // free the handle used by league name field
  FldSetTextHandle((FieldPtr)UIGetObjectPtr(fieldID_leaguename), NULL);
  MemHandleFree(GLeagueNameHandle);
}


// Eventhandler for "Edit league" form.
Boolean EditLeagueFormEvtHandler(EventPtr event) {
  Boolean handled = false;
  Err     err;

  switch (event->eType) {

 // form is opened
  case frmOpenEvent:
    InitEditLeagueForm();
    FrmDrawForm(FrmGetActiveForm());
    handled = true;
    break;

 // form is closed
  case frmCloseEvent:
    FreeEditLeagueForm();
    break;

 // a button was pressed
  case ctlSelectEvent:
    switch(event->data.ctlSelect.controlID) {

      // OK -> try and edit the league data
      case btnID_OK:
        err = SubmitEditLeagueForm();
        switch(err) {
          case 0: // no error
            FreeEditLeagueForm();
            FrmReturnToForm(0);
            ShowLeague(GState.currentLeagueIndex, GState.mode, true);
            break;
          case createLeagueErr:
            FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_LEAGUE_MUST_HAVE_NAME, NULL, NULL);
            break;
          case dmErrInvalidDatabaseName:
          case dmErrAlreadyExists:
            FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_LEAGUE_INVALID_NAME, NULL, NULL);
            break;
          default: // all other errors (not handled at the moment)
            FreeEditLeagueForm();
            FrmReturnToForm(0);
        }
        handled = true;
        break;

      // Cancel -> just return to previous form
      case btnID_Cancel:
        FreeEditLeagueForm();
        FrmReturnToForm(0);
        handled = true;
        break;
    }
    break;
  }

  return handled;
}


//
// Edit teams
//

// Initializes "Edit team" form.
void InitEditTeamsForm() {
  Int8    i;
  ListPtr teamList;
  League  league = LeagueGetCurrent();
  Int16   sortedIndex[MAX_TEAMS];

  // get indexes for the teams, sorted alphabetically
  LeagueGetSortedTeamIndexes(&league, sortedIndex);

  // create an array of pointers to the team names
  for(i = 0; i < league.teamCount; i++) {
    GDialogTeamNames[i] = LeagueGetTeam(&league, sortedIndex[i]);
  }

  // give the pointer array to the list-control
  teamList = (ListPtr)UIGetObjectPtr(listID_teams);
  LstSetListChoices(teamList, GDialogTeamNames, league.teamCount);
  LstSetSelection(teamList, -1);
  // create and set handles for the team name and team points fields
  GTeamNameHandle = MemHandleNew(11);
  FldSetTextHandle((FieldPtr)UIGetObjectPtr(fieldID_teamname), GTeamNameHandle);
  GTeamPointsHandle = MemHandleNew(4);
  FldSetTextHandle((FieldPtr)UIGetObjectPtr(fieldID_teampoints), GTeamPointsHandle);
}


// Frees "Edit team" form.
void FreeEditTeamsForm() {
  Int8   i;
  League league = LeagueGetCurrent();

  // free the handles used by team name and team points fields
  FldSetTextHandle((FieldPtr)UIGetObjectPtr(fieldID_teamname), NULL);
  MemHandleFree(GTeamNameHandle);
  FldSetTextHandle((FieldPtr)UIGetObjectPtr(fieldID_teampoints), NULL);
  MemHandleFree(GTeamPointsHandle);

  // unlock the pointers to the team names
  for(i = 0; i < league.teamCount; i++) {
    MemPtrUnlock(GDialogTeamNames[i]);
  }
}


// Copies the selected team name to the team name text field.
// Also read the team points from the db and displays them.
void SelectTeam(Int16 listIndex) {
  Char    buf[4];
  Int16   teamIndex;
  Int16   sortedIndex[MAX_TEAMS];
  League  league = LeagueGetCurrent();

  // get indexes for the teams, sorted alphabetically
  LeagueGetSortedTeamIndexes(&league, sortedIndex);
  teamIndex = sortedIndex[listIndex];
  // get the points to a char buffer
  StrIToA(buf, LeagueGetTeamPoints(&league, teamIndex));

  // update the fieds
  UISetFieldText(fieldID_teamname, GDialogTeamNames[listIndex], true);
  UISetFieldText(fieldID_teampoints, buf, true);
}


// Saves the team name and the team points.
void SaveTeamData() {
  ListPtr teamList  = (ListPtr)UIGetObjectPtr(listID_teams);
  Char*   teamName  = FldGetTextPtr((FieldPtr)UIGetObjectPtr(fieldID_teamname));
  League  league    = LeagueGetCurrent();
  Int16   sortedIndex[MAX_TEAMS];
  Int16   teamIndex;
  Int8    points;
  Int16   selectedIndex = LstGetSelection(teamList);

  if (selectedIndex != noListSelection) {
    // get indexes for the teams, sorted alphabetically
    LeagueGetSortedTeamIndexes(&league, sortedIndex);
    teamIndex = sortedIndex[selectedIndex];
    // save the new name
    LeagueSaveTeam(&league, teamIndex, teamName);
    // save the points
    points = (Int8)UIGetFieldIntValue(fieldID_teampoints);
    LeagueSaveTeamPoints(&league, teamIndex, points);
  }

  // clear the text fields and set list selection to none
  UISetFieldText(fieldID_teamname, "", true);
  UISetFieldText(fieldID_teampoints, "", true);
  LstSetSelection(teamList, -1);
  // free and re-init form so teams are re-sorted
  FreeEditTeamsForm();
  InitEditTeamsForm();
  // re-draw the list
  LstEraseList(teamList);
  LstDrawList(teamList);
}


// Eventhandler for "Edit teams" form.
Boolean EditTeamsFormEvtHandler(EventPtr event) {
  Boolean handled = false;

  switch (event->eType) {

  // form is opened
  case frmOpenEvent:
    InitEditTeamsForm();
    FrmDrawForm(FrmGetActiveForm());
    handled = true;
    break;

 // form is closed
  case frmCloseEvent:
    FreeEditTeamsForm();
    break;

  // a team was tapped in the list
  case lstSelectEvent:
    SelectTeam(event->data.lstSelect.selection);
    handled = true;
    break;

  // a button was pressed
  case ctlSelectEvent:
    switch(event->data.ctlSelect.controlID) {

      // close the form
      case btnID_close:
        FreeEditTeamsForm();
        FrmReturnToForm(0);
        ShowLeague(GState.currentLeagueIndex, GState.mode, true);
        handled = true;
        break;

      // save the team name
      case btnID_save:
        SaveTeamData();
        handled = true;
        break;
    }
    break;
  }

  return handled;
}


//
// Edit fixtures
//

// Initializes "Edit fixtures" form.
void InitEditFixturesForm() {
  Int8    i;
  Char*   tempPtr;
  ListPtr hometeamList;
  ListPtr awayteamList;
  League  league = LeagueGetCurrent();
  DateTimeType now;
  Int16   sortedIndex[MAX_TEAMS];

  // get indexes for the teams, sorted alphabetically
  LeagueGetSortedTeamIndexes(&league, sortedIndex);

  // create an array of pointers to the team names
  for(i = 0; i < league.teamCount; i++) {
    tempPtr = LeagueGetTeam(&league, sortedIndex[i]);
    GDialogTeamNames[i] = MemPtrNew(11);
    StrCopy(GDialogTeamNames[i], tempPtr);
    MemPtrUnlock(tempPtr);
    GMatchStrings[i] = MemPtrNew(35);
  }
  // give the pointer array to the list-controls
  hometeamList = (ListPtr)UIGetObjectPtr(popListID_hometeam);
  LstSetListChoices(hometeamList, GDialogTeamNames, league.teamCount);
  UISelectDropDown(popListID_hometeam, popTrigID_hometeam, 0);
  LstSetHeight(hometeamList, min(league.teamCount, 10));
  awayteamList = (ListPtr)UIGetObjectPtr(popListID_awayteam);
  LstSetListChoices(awayteamList, GDialogTeamNames, league.teamCount);
  UISelectDropDown(popListID_awayteam, popTrigID_awayteam, 1);
  LstSetHeight(awayteamList, min(league.teamCount, 10));

  // set global variables
  GRound = league.roundCount;
  TimSecondsToDateTime(TimGetSeconds(), &now);
  GYear  = now.year;
  GMonth = now.month;
  GDay   = now.day;
}


// Frees "Edit fixtures" form.
void FreeEditFixturesForm() {
  Int8   i;
  League league = LeagueGetCurrent();

  // unlock the pointers to the team names
  for(i = 0; i < league.teamCount; i++) {
    MemPtrFree(GMatchStrings[i]);
    MemPtrFree(GDialogTeamNames[i]);
  }
}


// Redraws the form
void UpdateEditFixturesForm() {
  Int16   i;
  Char    buf[20];
  FormPtr form = FrmGetActiveForm();
  ListPtr matchList;
  Int16   matchesInRound[MAX_TEAMS];
  Int8    matchesInRoundCount;
  League  league = LeagueGetCurrent();

  // get all the matches for the current round
  matchesInRoundCount = LeagueGetMatchesInRound(&league, GRound, matchesInRound);
  for(i = 0; i < matchesInRoundCount; i++) {
    LeagueGetMatchString(&league, matchesInRound[i], GMatchStrings[i]);
  }

  // set matches for the list control
  matchList = (ListPtr)UIGetObjectPtr(listID_fixtures);
  LstSetListChoices(matchList, GMatchStrings, matchesInRoundCount);
  LstSetSelection(matchList, -1);

  // round label
  StrPrintF(buf, TEXT_MATCH_ROUND, GRound);
  UISetControlLabel(form, labelID_round, buf);

  // date control
  StrPrintF(buf, TEXT_EDIT_FIX_DATE, GDay, GMonth);
  UISetControlLabel(form, selectTrigID_date, buf);
}


// Called when 'Add' button is clicked
void AddMatch() {
  Match   match;
  ListPtr matchList = (ListPtr)UIGetObjectPtr(listID_fixtures);
  League  league = LeagueGetCurrent();
  Int16   sortedIndex[MAX_TEAMS];

  // get indexes for the teams, sorted alphabetically
  LeagueGetSortedTeamIndexes(&league, sortedIndex);

  // check that we don't add too many matches
  if(LstGetNumberOfItems(matchList) >= league.teamCount) {
    FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_FIX_CANNOT_ADD_MATCH, NULL, NULL);
  }

  else {
    // get data from form
    match.round     = GRound;
    match.dateYear  = GYear;
    match.dateMonth = GMonth;
    match.dateDay   = GDay;
    match.homeTeam  = sortedIndex[LstGetSelection((ListPtr)UIGetObjectPtr(popListID_hometeam))];
    match.awayTeam  = sortedIndex[LstGetSelection((ListPtr)UIGetObjectPtr(popListID_awayteam))];
    match.homeGoals = -1;
    match.awayGoals = -1;
    match.isPlayed  = false;

    // create the new match
    if(match.homeTeam != match.awayTeam) {
      LeagueAddMatch(&league, &match, -1);
    }
  }

}


// Called when 'Delete match' is selected
void DeleteMatch() {
  Int16  selectedIndex;
  Int16  matchesInRound[MAX_TEAMS];
  League league = LeagueGetCurrent();

  LeagueGetMatchesInRound(&league, GRound, matchesInRound);
  selectedIndex = LstGetSelection((ListPtr)UIGetObjectPtr(listID_fixtures));

  if(selectedIndex == noListSelection) {
    FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_FIX_NO_MATCH_SELECTED, NULL, NULL);
  }
  else {
    LeagueDeleteMatch(&league, GRound, matchesInRound[selectedIndex]);
  }
}


// Called when 'Delete round' is selected
void DeleteRound() {
  Int16  matchesInRound[MAX_TEAMS];
  Int8   matchesInRoundCount;
  League league = LeagueGetCurrent();

  matchesInRoundCount = LeagueGetMatchesInRound(&league, GRound, matchesInRound);

  if(GRound < league.roundCount) {
    FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_FIX_ONLY_DELETE_LAST_ROUND, NULL, NULL);
  }
  else if(GRound == 1) {
    FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_FIX_CANNOT_DELETE_FIRST_ROUND, NULL, NULL);
  }
  else if(matchesInRoundCount > 0) {
    FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_FIX_ONLY_DELETE_EMPTY_ROUND, NULL, NULL);
  }
  else {
    LeagueDeleteRound(&league, GRound);
    GRound--;
  }

}


// Eventhandler for "Edit fixtures" form.
Boolean EditFixturesFormEvtHandler(EventPtr event) {
  Boolean handled      = false;
  Boolean updateScreen = false;
  League  league;

  switch (event->eType) {

  // form is opened
  case frmOpenEvent:
    InitEditFixturesForm();
    updateScreen = true;
    handled = true;
    break;

 // form is closed
  case frmCloseEvent:
    FreeEditFixturesForm();
    break;

 // form need to be updated
  case frmUpdateEvent:
    updateScreen = true;
    handled = true;
    break;

 // one of the scrollbuttons was pressed
  case keyDownEvent:
    switch(event->data.keyDown.chr) {
      case pageUpChr :
        GRound = max(GRound-1, 1);
        updateScreen = true;
        handled = true;
        break;
      case pageDownChr:
        league = LeagueGetCurrent();
        GRound = min(GRound+1, league.roundCount);
        updateScreen = true;
        handled = true;
        break;
    }
    break;

  // a button was pressed
  case ctlSelectEvent:
    switch(event->data.ctlSelect.controlID) {

      // close the form
      case btnID_close:
        FreeEditFixturesForm();
        FrmReturnToForm(0);
        handled = true;
        break;
      // add a match
      case btnID_addMatch:
        AddMatch();
        updateScreen = true;
        handled = true;
        break;
      // go to previous round
      case btnID_roundBack:
        GRound = max(GRound-1, 1);
        handled = true;
        updateScreen = true;
        break;
      // go to next round
      case btnID_roundFwd:
        league = LeagueGetCurrent();
        GRound = min(GRound+1, league.roundCount);
        handled = true;
        updateScreen = true;
        break;
      // date-control tapped
      case selectTrigID_date:
        SelectDay(selectDayByDay, &GMonth, &GDay, &GYear, TEXT_MATCH_SELECT_MATCH_DAY);
        updateScreen = true;
        break;
    }
    break;

  // a menu item was selected
  case menuEvent:
    switch (event->data.menu.itemID) {
      // add round
      case menuitemID_fixtures_addround:
        league = LeagueGetCurrent();
        LeagueAddRound(&league);
        GRound = league.roundCount;
        updateScreen = true;
        break;
      // delete round
      case menuitemID_fixtures_deleteround:
        DeleteRound();
        updateScreen = true;
        break;
      // delete match
      case menuitemID_fixtures_deletematch:
        DeleteMatch();
        updateScreen = true;
        break;
    }
    break;

   // the screen was tapped
   // used when the user wants to jump to a round
    case penDownEvent:
      if(((event->screenX >= 54) && (event->screenX <= 90)) &&
         ((event->screenY >= 15) && (event->screenY <= 25))) {
        FrmPopupForm(formID_selectRound);
        handled = true;
      }
    break;

  }

  //  redraw the screen if necessary
  if(updateScreen) {
    UpdateEditFixturesForm();
    FrmDrawForm(FrmGetActiveForm());
    // underline the round label
    WinDrawGrayLine(54, 25, 90, 25);
  }

  return handled;
}


//
// Select round
//

// Draws a cell in the table. This is a callback function which is called
// when the table is redrawn.
void DrawRoundCell(void* tablePtr, Int16 row, Int16 column, RectanglePtr bounds) {
  Char   buf[5];
  Int16  round = (row*8) + column+1;
  League league = LeagueGetCurrent();

  if(round <= league.roundCount) {
    StrIToA(buf, round);
    UIDrawCharsCenter(buf, bounds->topLeft.x+(bounds->extent.x/2), bounds->topLeft.y);
  }
}


// Initializes "Select round" form.
void InitSelectRoundForm() {
  Int16    i;
  TablePtr tablePtr = (TablePtr)UIGetObjectPtr(tableID_rounds);

  UIInitCustomDrawTable(tableID_rounds, 8, 8, DrawRoundCell);
  for(i = 0; i < 8; i++) {
    TblSetRowHeight(tablePtr, i, 14);
  }
}


// Frees "Select round" form.
void FreeSelectRoundForm() {
}


// Eventhandler for "Select round" form.
Boolean SelectRoundFormEvtHandler(EventPtr event) {
  Boolean handled = false;
  Int16   row, col, round;
  UInt16  callingFormId;
  League  league;

  switch (event->eType) {

  // form is opened
  case frmOpenEvent:
    InitSelectRoundForm();
    FrmDrawForm(FrmGetActiveForm());
    handled = true;
    break;

 // form is closed
  case frmCloseEvent:
    FreeSelectRoundForm();
    break;

  // a button was pressed
  case ctlSelectEvent:
    switch(event->data.ctlSelect.controlID) {

      // close the form
      case btnID_Cancel:
        FreeSelectRoundForm();
        FrmReturnToForm(0);
        handled = true;
        break;

    }
    break;

  // the screen was tapped
  // used when the user selects a round
  case penDownEvent:
    if(((event->screenX >= 2) && (event->screenX < 158)) &&
       ((event->screenY >= 18) && (event->screenY < 130))) {

      league = LeagueGetCurrent();
      row    = (event->screenY-18) / 14;
      col    = (event->screenX-2) / 19;
      round  = (row*8) + col+1;

      // make sure round is sane
      if(round <= league.roundCount) {
        // free resources and close form
        FreeSelectRoundForm();
        FrmReturnToForm(0);
        // update round and redraw the calling form
        callingFormId = FrmGetFormId(FrmGetFirstForm());
        if(callingFormId == formID_matches) {
          GState.Match.round = round;
          FrmUpdateForm(formID_matches, 0);
        }
        else if(callingFormId == formID_editFixtures) {
          GRound = round;
          FrmUpdateForm(formID_editFixtures, 0);
        }
      }
      handled = true;
    }
    break;
  }

  return handled;
}


//
// About
//

// Eventhandler for "About" form.
Boolean AboutFormEvtHandler(EventPtr event) {
  Boolean handled = false;

  switch (event->eType) {

  // form is opened
  case frmOpenEvent:
    FrmDrawForm(FrmGetActiveForm());
    handled = true;
    break;

  // a button was pressed
  case ctlSelectEvent:
    FrmReturnToForm(0);
    handled = true;
    break;
  }

  return handled;
}
