/*
 * TeamStats by Mats Bovin (mats@mbovin.com)
 * v1.3.0 September 2002
 * Copyright (c) 2002 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <PalmOS.h>
#include "Funcs.h"


//
// Database functions
//

// Returns the application info block for a database
// by its type and creator. Returns NULL if the database
// doesn't have an app info block or if an error ocurrs.
// The returned value is a locked pointer which must be
// unlocked later.
MemPtr GetAppInfoBlock(UInt32 type, UInt32 creator)
{
  DmOpenRef dbRef;
  LocalID   dbId;
  UInt16    cardNo;
  LocalID   appInfoId;
  MemPtr    appInfoPtr = NULL;

  dbRef = DmOpenDatabaseByTypeCreator(type, creator, dmModeReadOnly);
  if (dbRef != 0) {
    DmOpenDatabaseInfo(dbRef, &dbId, NULL, NULL, &cardNo, NULL);
    DmDatabaseInfo(cardNo, dbId, NULL, NULL, NULL,
        NULL, NULL, NULL, NULL, &appInfoId, NULL, NULL, NULL);

    if (appInfoId != NULL) {
      appInfoPtr = MemLocalIDToLockedPtr(appInfoId, cardNo);
    }

    DmCloseDatabase(dbRef);
  }

  return appInfoPtr;
}


// Creates the application info block for a database
// by its type and creator. Returns NULL if an error ocurrs.
// The returned value is a locked pointer which must be
// unlocked later.
MemPtr CreateAppInfoBlock(UInt32 type, UInt32 creator, UInt32 size) {
  DmOpenRef dbRef;
  LocalID   dbId;
  UInt16    cardNo;
  LocalID   appInfoId;
  MemPtr    appInfoPtr = NULL;
  MemHandle handle;

  dbRef = DmOpenDatabaseByTypeCreator(type, creator, dmModeReadOnly);
  if (dbRef != 0) {
    handle    = DmNewHandle(dbRef, size);
    appInfoId = MemHandleToLocalID(handle);

    DmOpenDatabaseInfo(dbRef, &dbId, NULL, NULL, &cardNo, NULL);
    DmSetDatabaseInfo(cardNo, dbId, NULL, NULL, NULL,
      NULL, NULL, NULL, NULL, &appInfoId, NULL, NULL, NULL);

    appInfoPtr = MemLocalIDToLockedPtr(appInfoId, cardNo);
    DmCloseDatabase(dbRef);
  }

  return appInfoPtr;
}


//
// Conversion functions
//

// Convert double to string with the specified number of decimals.
// This function is based on code by David W. Bray (http://www.40-below.com/palm).
void DoubleToStr(double flpNumber, Char* str, Int8 numFractDigits) {
  Int32   longNumber, flpIP;
  double  zeros, round;
  Int16   i, remainder, strLen, startPos;
  Boolean isNeg = false;

  if (flpNumber < 0.0) {
    isNeg = true;
    flpNumber = -flpNumber;
  }

  zeros = 1.0;
  for (i=0; i<numFractDigits; i++) {
    zeros *= 10;
  }

  round = 0.5/zeros;

  flpNumber += round;
  if(flpNumber < 1) {
    flpIP = 0;
  }
  else {
    flpIP = (Int32)flpNumber;
  }
  flpNumber = flpNumber - (Int32)flpIP;

  if(isNeg) {
    str[0] = '-';
    startPos = 1;
  }
  else {
    startPos = 0;
  }

  StrIToA(&str[startPos], (Int32) flpIP);
  strLen = StrLen(str); // put in the decimal point and terminate the string
  str[strLen] = '.';
  str[numFractDigits+strLen+1] = '\0';

  longNumber = flpNumber * zeros; // fractional part

  for (i=numFractDigits+strLen; i>strLen; i--) {  // convert the integer part
    remainder = longNumber % 10;
    str[i] = remainder + 0x30;
    longNumber /= 10;
  }

}


// Convert double to a percent string with the specified number of decimals.
void DoubleToPercentStr(double flpNumber, Char* str, Int8 numFractDigits) {
  DoubleToStr(flpNumber * 100, str, numFractDigits);
  StrCat(str, "%");
}


//
// UIFunctions
//

// Returns a pointer to the specified object. The object is assumed
// to be in the active form.
void* UIGetObjectPtr(UInt16 objID) {
  FormPtr form = FrmGetActiveForm();
  return FrmGetObjectPtr(form, FrmGetObjectIndex(form, objID));
}


// Sets the selection of 'listID' to the specified index. The label of that
// item is copied to 'triggerID'.
void UISelectDropDown(UInt16 listID, UInt16 triggerID, Int16 selectedIndex) {
  ListPtr    listPtr    = (ListPtr)UIGetObjectPtr(listID);
  ControlPtr triggerPtr = (ControlPtr)UIGetObjectPtr(triggerID);

  LstSetSelection(listPtr, selectedIndex);
  CtlSetLabel(triggerPtr, LstGetSelectionText(listPtr, selectedIndex));
}


// Changes the label of the specified control.
void UISetControlLabel(FormPtr formPtr, UInt16 ctlID, Char* newLabel) {
  ControlPtr ctlPtr   = (ControlPtr)UIGetObjectPtr(ctlID);
  Char*      labelPtr = (Char*)CtlGetLabel(ctlPtr);

  FrmHideObject(formPtr, FrmGetObjectIndex(formPtr, ctlID));
  StrCopy(labelPtr, newLabel);
  FrmShowObject(formPtr, FrmGetObjectIndex(formPtr, ctlID));
}


// Initializes a table with the specified number of rows and columns.
// All table cells will have the style 'customTableItem' and use the
// function 'drawCallback' to draw the themselves.
void UIInitCustomDrawTable(UInt16 tableID, Int16 rows, Int16 columns, TableDrawItemFuncPtr drawCallback) {
  Int16 row;
  Int16 col;
  TablePtr tablePtr = (TablePtr)UIGetObjectPtr(tableID);

  for(col = 0; col < columns; col++) {
    for(row = 0; row < rows; row++) {
      TblSetItemStyle(tablePtr, row, col, customTableItem);
      if(col == 0) {
        TblSetRowSelectable(tablePtr, row, false);
      }
    }
    TblSetColumnUsable(tablePtr, col, true);
    TblSetCustomDrawProcedure(tablePtr, col, drawCallback);
  }

}


// Changes the text of the specified field. The field must already have
// a valid text handle. Also, the new text cannot be longer than the old one.
void UISetFieldText(UInt16 fieldID, Char* newText, Boolean redraw) {
  Char*     charPtr;
  FieldPtr  fieldPtr = (FieldPtr)UIGetObjectPtr(fieldID);
  // get the old handle...
  MemHandle textHandle = FldGetTextHandle(fieldPtr);
  // ...and disconnect it from the field
  FldSetTextHandle(fieldPtr, NULL);
  // copy the new text to the handle
  charPtr = MemHandleLock(textHandle);
  StrCopy(charPtr, newText);
  MemHandleUnlock(textHandle);
  // set the handle for the field
  FldSetTextHandle(fieldPtr, textHandle);
  // redraw the field, if requested
  if(redraw) {
    FldDrawField(fieldPtr);
  }
}


// Returns the value of a field as an integer. Is best used with numeric fields.
// 0 (zero) is returned if the field is empty.
Int16 UIGetFieldIntValue(UInt16 fieldID) {
  Int16     result     = 0;
  FieldPtr  fieldPtr   = (FieldPtr)UIGetObjectPtr(fieldID);
  MemHandle textHandle = FldGetTextHandle(fieldPtr);

  if(textHandle != NULL) {
    Char* charPtr = MemHandleLock(textHandle);

    if(StrLen(charPtr) > 0) {
      result = StrAToI(charPtr);
    }

    MemHandleUnlock(textHandle);
  }

  return result;
}


// Draw a string left justified
void UIDrawCharsLeft(const Char *chars, Coord x, Coord y) {
  WinDrawChars(chars, StrLen(chars), x, y);
}

// Draw a string right justified
void UIDrawCharsRight(const Char *chars, Coord x, Coord y) {
  WinDrawChars(chars, StrLen(chars), x - FntCharsWidth(chars, StrLen(chars)), y);
}

// Draw a string centered
void UIDrawCharsCenter(const Char *chars, Coord x, Coord y) {
  WinDrawChars(chars, StrLen(chars), x - (FntCharsWidth(chars, StrLen(chars))/2), y);
}


// Moves the scrollbar to the specified position.
void UISetScrollBarPos(UInt16 scrollBarID, Int16 newPos) {
  Int16 minValue, maxValue, pageSize, dummy;
  ScrollBarPtr scrollPtr = (ScrollBarPtr)UIGetObjectPtr(scrollBarID);
  SclGetScrollBar(scrollPtr, &dummy, &minValue, &maxValue, &pageSize);
  newPos = (newPos > maxValue) ? maxValue : newPos;
  newPos = (newPos < minValue) ? minValue : newPos;
  SclSetScrollBar(scrollPtr, newPos, minValue, maxValue, pageSize);
}
