/*
 * TeamStats by Mats Bovin (mats@mbovin.com)
 * v1.3.0 September 2002
 * Copyright (c) 2002 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <PalmOS.h>
#include "TeamStats.h"
#include "rcp.h"
#include "League.h"
#include "Funcs.h"
#include "Text.h"
#include <HostControl.h>


//
// Global variables
//

// The state of the application
AppState GState;
// All the names of the leagues (used in drop-down)
Char* GLeagueNames[MAX_LEAGUES];

// The currently open league database
DmOpenRef GCurrentDB = NULL;
// Info about the league databases
LeagueDBInfo GLeagueDB[MAX_LEAGUES];


//
// Forward declarations
//
void LookForDatabases();
void AddLeagueInfo(Char* leagueName, LocalID dbID, UInt16 cardNo);


//
// Functions
//


// Callback function used when beaming database.
Err WriteDBData(const void* dataP, UInt32* sizeP, void* userDataP) {
  Err err;

  *sizeP = ExgSend((ExgSocketPtr)userDataP, (void*)dataP, *sizeP, &err);

  return err;
}


// Beams the current the database.
void BeamCurrentDatabase() {
  ExgSocketType exgSocket;
  Err           err;
  Char          filename[40];

  MemSet(&exgSocket, sizeof(exgSocket), 0);
  StrCopy(filename, GLeagueNames[GState.currentLeagueIndex]);
  StrCat(filename, ".pdb");
  exgSocket.description = GLeagueNames[GState.currentLeagueIndex];
  exgSocket.name        = filename;

  err = ExgPut(&exgSocket);
  if(!err) {
    err = ExgDBWrite(WriteDBData, &exgSocket, NULL,
            GLeagueDB[GState.currentLeagueIndex].localID,
            GLeagueDB[GState.currentLeagueIndex].cardNo);
    err = ExgDisconnect(&exgSocket, err);
  }

}


// Opens a database to use as the current league database.
void OpenDatabase(UInt16 cardNo, LocalID dbID) {
  GCurrentDB = DmOpenDatabase(cardNo, dbID, dmModeReadWrite);
}


// Closes the current database.
void CloseCurrentDatabase() {
  if(GCurrentDB) {
    DmCloseDatabase(GCurrentDB);
    GCurrentDB = NULL;
  }
}


// Creates a new database with the specified name and league information.
// Returns any errors that occured when the db was created.
Err CreateDatabase(Char* dbName, League *league) {
  Int8    i;
  Err     err;
  LocalID dbID;
  UInt16  index = 0;
  UInt16  dbAttrs;

  err = DmCreateDatabase(0, dbName, 'mbTS', 'Data', false);
  if(err) {
    return err;
  }

  dbID    = DmFindDatabase(0, dbName);
  // set the backup flag so the db is copied to the desktop PC during sync
  dbAttrs = dmHdrAttrBackup;
  DmSetDatabaseInfo(0, dbID, NULL, &dbAttrs, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);

  // open the db
  league->dbRef = DmOpenDatabase(0, dbID, dmModeReadWrite);
  // set some default values
  league->dbVersion        = 1;
  league->matchCount       = 0;
  league->playedMatchCount = 0;
  league->roundCount       = 0;
  league->line[0] = league->line[1] = league->line[2] = league->line[3] = -1;

  // create the first record...
  DmNewRecord(league->dbRef, &index, 14);
  DmReleaseRecord(league->dbRef, index, true);
  // ... so we can store the league info there.
  LeagueSave(league);

  // create records for the teams in the league and store a default name for each team
  index++;
  for(i = 0; i < league->teamCount; i++) {
    Char buf[10];
    StrPrintF(buf, TEXT_EDIT_LEAGUE_TEAM, i+1);
    DmNewRecord(league->dbRef, &index, 12);
    DmReleaseRecord(league->dbRef, index, true);
    LeagueSaveTeam(league, i, buf);
    LeagueSaveTeamPoints(league, i, 0);
    index++;
  }

  // add the first (empty) round
  LeagueAddRound(league);

  DmCloseDatabase(league->dbRef);

  // add info about the new db to global variables
  AddLeagueInfo(dbName, dbID, 0);

  // return no error
  return 0;
}


// Deletes the currently open database.
void DeleteCurrentDatabase() {
  CloseCurrentDatabase();
  DmDeleteDatabase(GLeagueDB[GState.currentLeagueIndex].cardNo, GLeagueDB[GState.currentLeagueIndex].localID);
  // this call makes sure that the deleted database is removed from the global variables
  // containing db info and league name.
  LookForDatabases();
}


// Edits the currently open database. The name of the db can be altered
// as well as the data in the league structure.
// Returns any errors that occured when the db name was altered.
Err EditCurrentDatabase(Char* newDBName, League *league) {
  Err err = 0;

  // change the db name if it's been altered
  if(StrCompare(newDBName, GLeagueNames[GState.currentLeagueIndex])) {
    err = DmSetDatabaseInfo(GLeagueDB[GState.currentLeagueIndex].cardNo,
                            GLeagueDB[GState.currentLeagueIndex].localID,
                            newDBName, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
  }
  // save the league data and update the league name in the drop-down list
  if(!err) {
    LeagueSave(league);
    StrCopy(GLeagueDB[GState.currentLeagueIndex].name, newDBName);
  }

  return err;
}


// Initializes the top toolbar of the form, ie the mode buttons and league selector.
// This top part is equal for all the mode forms.
void InitTop(Int8 mode) {
  UInt16     btnID = 0;
  ControlPtr compareCtl;
  ListPtr    leagueList;

  switch(mode) {
    case MODE_TABLE:   btnID = btnID_table;   break;
    case MODE_MATCHES: btnID = btnID_matches; break;
    case MODE_TEAM:    btnID = btnID_team;    break;
    case MODE_LEAGUE:  btnID = btnID_league;  break;
  }
  // make the correct mode button selected.
  compareCtl = (ControlPtr)UIGetObjectPtr(btnID);
  CtlSetValue(compareCtl, 1);

  // populate the league drop-down
  leagueList = (ListPtr)UIGetObjectPtr(popListID_league);
  LstSetListChoices(leagueList, GLeagueNames, GState.leagueCount);
  LstSetHeight(leagueList, min(GState.leagueCount, 10));
  // select the current league
  UISelectDropDown(popListID_league, popTrigID_league, GState.currentLeagueIndex);
}


// Sets default state for the current league.
void InitLeague(Int8 startupMode) {
  League league = LeagueGetCurrent();
 // app settings
  GState.mode = startupMode;
 // table form settings
  GState.Table.matches    = 0;
  GState.Table.rounds     = 0;
  GState.Table.roundCount = 5;
  GState.Table.scrollPos  = 0;
 // match form settings
  GState.Match.round     = LeagueGetCurrentRound(&league);
  GState.Match.scrollPos = 0;
 // team form settings
  GState.Team.team      = 0;
  GState.Team.matches   = 0;
  GState.Team.mode      = 0;
  GState.Team.scrollPos = 0;
}


// Switches to the specified league and mode.
void ShowLeague(Int8 leagueIndex, Int8 startupMode, Boolean initState) {
  UInt16 formID = 0;

  // close the current db and open the new one
  CloseCurrentDatabase();
  OpenDatabase(GLeagueDB[leagueIndex].cardNo, GLeagueDB[leagueIndex].localID);
  GState.currentLeagueIndex = leagueIndex;
  // initialize the league...
  if (initState) {
    InitLeague(startupMode);
  }
  // ...and show the desired form
  switch(startupMode) {
    case MODE_TABLE:   formID = formID_table; break;
    case MODE_MATCHES: formID = formID_matches; break;
    case MODE_TEAM:    formID = formID_team; break;
    case MODE_LEAGUE:  formID = formID_league; break;
  }
  FrmGotoForm(formID);
}


// Adds league info to the global data structures.
void AddLeagueInfo(Char* leagueName, LocalID dbID, UInt16 cardNo) {
  // store the name (for use in drop-down)
  StrCopy(GLeagueDB[GState.leagueCount].name, leagueName);
  GLeagueNames[GState.leagueCount] = GLeagueDB[GState.leagueCount].name;
  // store the location of the db
  GLeagueDB[GState.leagueCount].localID = dbID;
  GLeagueDB[GState.leagueCount].cardNo  = cardNo;
  // increment the league count
  GState.leagueCount++;
}


// Looks for league databases. The localID of each found league is stored in a
// globally available data structure.
void LookForDatabases() {
  Err               err;
  DmSearchStateType state;
  UInt16            cardNo;
  LocalID           currentDBID = 0;

  GState.leagueCount = 0;
  // look for the first league database
  err = DmGetNextDatabaseByTypeCreator(true, &state, 'Data', 'mbTS', false, &cardNo, &currentDBID);

  while (!err && currentDBID) {
    Char leagueName[32];
    DmDatabaseInfo(cardNo, currentDBID, leagueName, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
    // store league info (name, database localtion) in a global data structure
    AddLeagueInfo(leagueName, currentDBID, cardNo);
    // search for the next db
    err = DmGetNextDatabaseByTypeCreator(false, &state, 'Data', 'mbTS', false, &cardNo, &currentDBID);
  }

}


// Copy the current state (GState) to the supplied structure
// (which is later stored in the app info block).
void FillAppInfoBlock(AppInfoBlock* appInfoPtr) {
  appInfoPtr->version = 1;
  StrCopy(appInfoPtr->dbName, GLeagueNames[GState.currentLeagueIndex]);
  appInfoPtr->mode            = GState.mode;
  appInfoPtr->tableMatches    = GState.Table.matches;
  appInfoPtr->tableRounds     = GState.Table.rounds;
  appInfoPtr->tableRoundCount = GState.Table.roundCount;
  appInfoPtr->tableScrollPos  = GState.Table.scrollPos;
  appInfoPtr->matchRound      = GState.Match.round;
  appInfoPtr->matchScrollPos  = GState.Match.scrollPos;
  appInfoPtr->teamTeam        = GState.Team.team;
  appInfoPtr->teamMatches     = GState.Team.matches;
  appInfoPtr->teamMode        = GState.Team.mode;
  appInfoPtr->teamScrollPos   = GState.Team.scrollPos;
}


// Copies data from the supplied data structure (which has been read
// from the app info block) to the the current state structure GState.
// Returns false if the current league isn't found (ie the league has
// been deleted).
Boolean InitAppState(AppInfoBlock* appInfoPtr) {
  Int16 i = 0;

  // find the league that should be the current league
  while ((i < GState.leagueCount) && StrCompare(GLeagueNames[i], appInfoPtr->dbName))
    i++;

  // no league found -> exit from function
  if (i == GState.leagueCount) {
    return false;
  }

  GState.currentLeagueIndex = i;
  GState.mode               = appInfoPtr->mode;
  GState.Table.matches      = appInfoPtr->tableMatches;
  GState.Table.rounds       = appInfoPtr->tableRounds;
  GState.Table.roundCount   = appInfoPtr->tableRoundCount;
  GState.Table.scrollPos    = appInfoPtr->tableScrollPos;
  GState.Match.round        = appInfoPtr->matchRound;
  GState.Match.scrollPos    = appInfoPtr->matchScrollPos;
  GState.Team.team          = appInfoPtr->teamTeam;
  GState.Team.matches       = appInfoPtr->teamMatches;
  GState.Team.mode          = appInfoPtr->teamMode;
  GState.Team.scrollPos     = appInfoPtr->teamScrollPos;

  return true;
}


// Saves the current state of application to the application
// info block. Called when the user exits from the application.
void SaveAppState() {
  AppInfoBlock* appInfoPtr;
  AppInfoBlock  appInfo;

  if (GState.leagueCount > 0) {
    appInfoPtr = (AppInfoBlock *)GetAppInfoBlock('appl', 'mbTS');

    if (appInfoPtr == NULL) {
      appInfoPtr = (AppInfoBlock *)CreateAppInfoBlock('appl', 'mbTS', sizeof(AppInfoBlock));
    }

    FillAppInfoBlock(&appInfo);
    DmWrite(appInfoPtr, 0, &appInfo, sizeof(AppInfoBlock));

    MemPtrUnlock(appInfoPtr);
  }
}


// Reads saved state from the application info block. Called when
// the application starts.
Boolean ReadAppState() {
  AppInfoBlock* appInfoPtr;
  Boolean       appInfoFound = false;

  appInfoPtr = (AppInfoBlock *)GetAppInfoBlock('appl', 'mbTS');

  if(appInfoPtr != NULL) {
    appInfoFound = InitAppState(appInfoPtr);
    MemPtrUnlock(appInfoPtr);
  }

  return appInfoFound;
}


// Event handler for the "no league"-form.
Boolean NoLeagueFormEvtHandler(EventPtr event) {
  FormPtr form    = FrmGetActiveForm();
  Boolean handled = false;

  switch (event->eType) {
    // form is opened
    case frmOpenEvent:
      FrmDrawForm(form);
      handled = true;
      break;
  }

  return handled;
}


// The application event handler. Used for launching the different forms.
Boolean ApplicationHandleEvent(EventPtr event) {
  Boolean handled = false;

  if(event->eType == frmLoadEvent) {
    UInt16  formId = event->data.frmLoad.formID;
    FormPtr form   = FrmInitForm(formId);
    FrmSetActiveForm(form);

    switch(formId) {
      case formID_noleague:
        FrmSetEventHandler(form, (FormEventHandlerPtr)NoLeagueFormEvtHandler);
        handled = true;
        break;
      case formID_table:
        FrmSetEventHandler(form, (FormEventHandlerPtr)TableFormEvtHandler);
        handled = true;
        break;
      case formID_matches:
        FrmSetEventHandler(form, (FormEventHandlerPtr)MatchFormEvtHandler);
        handled = true;
        break;
      case formID_team:
        FrmSetEventHandler(form, (FormEventHandlerPtr)TeamFormEvtHandler);
        handled = true;
        break;
      case formID_league:
        FrmSetEventHandler(form, (FormEventHandlerPtr)LeagueFormEvtHandler);
        handled = true;
        break;
      case formID_createLeague:
        FrmSetEventHandler(form, (FormEventHandlerPtr)CreateLeagueFormEvtHandler);
        handled = true;
        break;
      case formID_editLeague:
        FrmSetEventHandler(form, (FormEventHandlerPtr)EditLeagueFormEvtHandler);
        handled = true;
        break;
      case formID_editTeams:
        FrmSetEventHandler(form, (FormEventHandlerPtr)EditTeamsFormEvtHandler);
        handled = true;
        break;
      case formID_editFixtures:
        FrmSetEventHandler(form, (FormEventHandlerPtr)EditFixturesFormEvtHandler);
        handled = true;
        break;
      case formID_selectRound:
        FrmSetEventHandler(form, (FormEventHandlerPtr)SelectRoundFormEvtHandler);
        handled = true;
        break;
      case formID_about:
        FrmSetEventHandler(form, (FormEventHandlerPtr)AboutFormEvtHandler);
        handled = true;
        break;
    }
  }

  if(event->eType == menuEvent) {
    UInt16 buttonIndex;
    UInt32 romversion;

    // send an update event to the match form so it
    // saves any unsaved results before we continue with
    // the menu handling
    FrmUpdateForm(formID_matches, 0);

    switch (event->data.menu.itemID) {
      case menuitemID_main_about:
      case menuitemID_noleague_about:
        FrmPopupForm(formID_about);
        break;
      case menuitemID_main_create:
      case menuitemID_noleague_create:
        // display "Create league" dialog
        FrmPopupForm(formID_createLeague);
        break;
      case menuitemID_main_delete:
        // display "Really delete?" confirmation
        buttonIndex = FrmCustomAlert(alertID_deleteLeague, GLeagueNames[GState.currentLeagueIndex], NULL, NULL);
        if(buttonIndex == 0) { // yes was clicked
          // delete the current db
          DeleteCurrentDatabase();
          // show table form for the first available database...
          if(GState.leagueCount > 0) {
            ShowLeague(0, MODE_TABLE, true);
          }
          // ...otherwise show "no league form"
          else {
            FrmGotoForm(formID_noleague);
          }
        }
        break;
      case menuitemID_main_beam:
        // check that we're on an OS version that supports beaming
        FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romversion);
        if(romversion >= 0x03003000) {
          BeamCurrentDatabase();
        }
        else {
          FrmCustomAlert(alertID_errGeneral, TEXT_MAIN_BEAMING_NOT_SUPPORTED, NULL, NULL);
        }
        break;
      case menuitemID_main_league:
        // display "Edit league" dialog
        FrmPopupForm(formID_editLeague);
        break;
      case menuitemID_main_teams:
        // display "Edit teams" dialog
        FrmPopupForm(formID_editTeams);
        break;
      case menuitemID_main_fixtures:
        // display "Edit fixtures" dialog
        FrmPopupForm(formID_editFixtures);
        break;
    }
    //handled = true;
  }

  return handled;
}


// The main event loop of the application.
void EventLoop() {
  EventType event;
  Err       err;

  do {
    EvtGetEvent(&event, 200);

    if(SysHandleEvent(&event))
      continue;
    if(MenuHandleEvent(NULL, &event, &err))
      continue;
    if(ApplicationHandleEvent(&event))
      continue;
    FrmDispatchEvent(&event);
  } while(event.eType != appStopEvent);
}


// Cleans up when application is about to finish.
void StopApplication() {
  FrmCloseAllForms();
  SaveAppState();
  CloseCurrentDatabase();
}


// Initializes the app by looking for league databases and
// then launching the main form.
void StartApplication() {
  Boolean appStateFound;
  Int8    dbIndex = 0;
  Int8    mode    = MODE_TABLE;

  LookForDatabases();
  // read the saved app state and start the application
  // where it was finished
  if(GState.leagueCount > 0) {
    appStateFound = ReadAppState();
    if (appStateFound) {
      dbIndex = GState.currentLeagueIndex;
      mode    = GState.mode;
    }
    ShowLeague(dbIndex, mode, !appStateFound);
  }
  // or show "no league form" if no leagues are available
  else {
    FrmGotoForm(formID_noleague);
  }
}

/*
void TraceMem() {
  UInt16 i;
  UInt16 numHeaps;
  UInt32 heapSize;
  UInt32 freeBytes;
  UInt32 maxSize;

  numHeaps = MemNumHeaps(0);
  HostTraceOutputTL(appErrorClass, "num heaps %hu", numHeaps);

  for(i = 0; i < numHeaps; i++) {
    UInt16 heapID = MemHeapID(0, i);
    heapSize = MemHeapSize(heapID);
    MemHeapFreeBytes(heapID, &freeBytes, &maxSize);
    HostTraceOutputTL(appErrorClass, "heap %hu, size: %lu, free: %lu, max: %lu", i, heapSize, freeBytes, maxSize);
  }
}
*/

// Starting point of application.
UInt32 PilotMain(UInt16 cmd, void* cmdPBP, UInt16 launchFlags) {

//  HostTraceInit();
//  HostTraceOutputTL(appErrorClass, "start");
//  TraceMem();

  if(cmd == sysAppLaunchCmdNormalLaunch) {
    // initialize the app
    StartApplication();
    // go into the main event loop
    EventLoop();
    // clean up and exit
    StopApplication();
  }

//  TraceMem();
//  HostTraceOutputTL(appErrorClass, "stop");
//  HostTraceClose();

  return 0;
}
