/*
 * TeamStats by Mats Bovin (mats@mbovin.com)
 * v1.3.0 September 2002
 * Copyright (c) 2002 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <PalmOS.h>
#include "TeamStats.h"
#include "rcp.h"
#include "League.h"
#include "Funcs.h"
#include "Text.h"


// Constants
#define VISIBLE_MATCHES 11

// Global variables
MemHandle GResultFieldHandle[VISIBLE_MATCHES*2];
Int16     GMatchesInRound[MAX_TEAMS];
Int8      GMatchesInRoundCount;


// Iterates through GMatchesInRound starting at 'arrayPos'. All indexes between
// 'oldIndex' and 'newIndex' are changed accordingly.
void CorrectMatchIndexes(Int16 oldMatchIndex, Int16 newMatchIndex, Int16 arrayPos) {
  Int16 i;

  if(oldMatchIndex > newMatchIndex) {
    for(i = arrayPos; i < min(GMatchesInRoundCount, VISIBLE_MATCHES); i++) {
      if((GMatchesInRound[i + GState.Match.scrollPos] <  oldMatchIndex) &&
         (GMatchesInRound[i + GState.Match.scrollPos] >= newMatchIndex)) {
        GMatchesInRound[i + GState.Match.scrollPos]++;
      }
    }
  }
  else if(oldMatchIndex < newMatchIndex) {
    for(i = arrayPos; i < min(GMatchesInRoundCount, VISIBLE_MATCHES); i++) {
      if((GMatchesInRound[i + GState.Match.scrollPos] >  oldMatchIndex) &&
         (GMatchesInRound[i + GState.Match.scrollPos] <= newMatchIndex)) {
        GMatchesInRound[i + GState.Match.scrollPos]--;
      }
    }
  }
}


// Saves the results of all edited matches.
void SaveChangedResults() {
  Int16  i;
  Int8   homeGoals;
  Int8   awayGoals;
  Int16  matchIndex;
  Int16  newMatchIndex;
  League league = LeagueGetCurrent();

  for(i = 0; i < min(GMatchesInRoundCount, VISIBLE_MATCHES); i++) {
    FieldPtr hgField = (FieldPtr)UIGetObjectPtr(fieldID_homeGoals + i);
    FieldPtr agField = (FieldPtr)UIGetObjectPtr(fieldID_awayGoals + i);
    // only save if at least one of the fields have been changed
    if(FldDirty(hgField) || FldDirty(agField)) {
      matchIndex = GMatchesInRound[i + GState.Match.scrollPos];
      // check that none of the fields are empty
      if ((FldGetTextLength((FieldPtr)UIGetObjectPtr(fieldID_homeGoals + i)) == 0) ||
          (FldGetTextLength((FieldPtr)UIGetObjectPtr(fieldID_awayGoals + i)) == 0)) {
        homeGoals  = -1;
        awayGoals  = -1;
      }
      else {
        homeGoals  = (Int8)UIGetFieldIntValue(fieldID_homeGoals + i);
        awayGoals  = (Int8)UIGetFieldIntValue(fieldID_awayGoals + i);
      }
      newMatchIndex = LeagueUpdateMatchResult(&league, matchIndex, homeGoals, awayGoals);
      // if the match changed index in the database we need to make sure that indexes
      // in GMatchesInRound that's between old and new index are changed accordingly.
      CorrectMatchIndexes(matchIndex, newMatchIndex, i+1);
      // clear the dirty bit
      FldSetDirty(hgField, false);
      FldSetDirty(agField, false);
    }
  }

}


// Displays a Select Date-dialog and saves the selected
// date for the specified match.
void ChangeDate(Int16 rowIndex) {
  Int16  matchIndex;
  League league = LeagueGetCurrent();
  Match  match;
  Int16  year, month, day;

  // get the current date of the match
  matchIndex = GMatchesInRound[rowIndex + GState.Match.scrollPos];
  match = LeagueGetMatch(&league, matchIndex);
  year  = match.dateYear;
  month = match.dateMonth;
  day   = match.dateDay;
  // display dialog
  SelectDay(selectDayByDay, &month, &day, &year, TEXT_MATCH_SELECT_MATCH_DAY);
  // save new date
  LeagueChangeMatchDate(&league, matchIndex, year, month, day);
}


// Initializes the form.
void InitMatchForm() {
  Int16  i;
  MemPtr resPtr;

  InitTop(MODE_MATCHES);

  // create text handles for the result input fields
  for(i = 0; i < VISIBLE_MATCHES*2; i++) {
    GResultFieldHandle[i] = MemHandleNew(3);
    resPtr = MemHandleLock(GResultFieldHandle[i]);
    StrCopy(resPtr, "");
    MemHandleUnlock(GResultFieldHandle[i]);
    FldSetTextHandle(UIGetObjectPtr(fieldID_homeGoals+i), GResultFieldHandle[i]);
  }
}


// Frees resources allocated for this form.
void FreeMatchForm() {
  Int16  i;

  // save the edited results
  SaveChangedResults();

  // free the text handles
  for(i = 0; i < VISIBLE_MATCHES*2; i++) {
    FldSetTextHandle(UIGetObjectPtr(fieldID_homeGoals+i), NULL);
    MemHandleFree(GResultFieldHandle[i]);
  }

}


// Updates the controls and data on the form.
void UpdateMatchForm() {
  Int16        i, dummy;
  Char         buf[20];
  ScrollBarPtr matchScl;
  FormPtr      form   = FrmGetActiveForm();
  League       league = LeagueGetCurrent();

  // save the edited results
  SaveChangedResults(&league);

  // get scrollbar position
  matchScl = (ScrollBarPtr)UIGetObjectPtr(scrollID_matches);
  SclGetScrollBar(matchScl, &GState.Match.scrollPos, &dummy, &dummy, &dummy);

  // sanity check of current round (it might have been deleted)
  if(GState.Match.round > league.roundCount) {
    GState.Match.round = LeagueGetCurrentRound(&league);
  }
  // get the matches in the current round
  GMatchesInRoundCount = LeagueGetMatchesInRound(&league, GState.Match.round, GMatchesInRound);

  // update the editable fields for results
  for(i = 0; i < VISIBLE_MATCHES; i++) {
    if(i < GMatchesInRoundCount) {
      Char homeBuf[3];
      Char awayBuf[3];
      Match match = LeagueGetMatch(&league, GMatchesInRound[i + GState.Match.scrollPos]);

      if(match.isPlayed) {
        StrIToA(homeBuf, match.homeGoals);
        StrIToA(awayBuf, match.awayGoals);
      }
      else {
        StrCopy(homeBuf, "");
        StrCopy(awayBuf, "");
      }

      UISetFieldText(fieldID_homeGoals + i, homeBuf, false);
      UISetFieldText(fieldID_awayGoals + i, awayBuf, false);

      FrmShowObject(form, FrmGetObjectIndex(form, fieldID_homeGoals + i));
      FrmShowObject(form, FrmGetObjectIndex(form, fieldID_awayGoals + i));
    }
    else {
      FrmHideObject(form, FrmGetObjectIndex(form, fieldID_homeGoals + i));
      FrmHideObject(form, FrmGetObjectIndex(form, fieldID_awayGoals + i));
    }
  }

  // show the round controls
  FrmShowObject(form, FrmGetObjectIndex(form, btnID_roundBack));
  StrPrintF(buf, TEXT_MATCH_ROUND, GState.Match.round);
  UISetControlLabel(form, labelID_round, buf);
  FrmShowObject(form, FrmGetObjectIndex(form, btnID_roundFwd));

  // update the scrollbar
  SclSetScrollBar(matchScl, GState.Match.scrollPos, 0, max(GMatchesInRoundCount-VISIBLE_MATCHES, 0), VISIBLE_MATCHES);
}


// Draws the form.
void DrawMatchForm() {
  Int16  i;
  Int16  y;
  Char   outBuf[50];
  Char*  outPtr;
  Match  match;
  League league = LeagueGetCurrent();
  // erase the draw window
  RectangleType eraseRect = { { 0, 39 } , { 125, 121 } };
  WinEraseRectangle(&eraseRect, 0);

  // underline the round label
  WinDrawGrayLine(55, 36, 86, 36); /*ToK: adjusted*/

  // draw the matches
  for(i = 0; i < min(GMatchesInRoundCount, VISIBLE_MATCHES); i++) {
    y = 39 + (i * 11);
    // get the match to draw
    match = LeagueGetMatch(&league, GMatchesInRound[i + GState.Match.scrollPos]);
    // date
    StrPrintF(outBuf, TEXT_MATCH_DATE, match.dateDay, match.dateMonth);
    WinDrawChars(outBuf, StrLen(outBuf), 0, y);
    WinDrawGrayLine(0, y+10, 24, y+10);
    // home team
    outPtr = LeagueGetTeam(&league, match.homeTeam);
    WinDrawChars(outPtr, StrLen(outPtr), 28, y);
    MemPtrUnlock(outPtr);
    // away team
    outPtr = LeagueGetTeam(&league, match.awayTeam);
    WinDrawChars(outPtr, StrLen(outPtr), 75, y);
    MemPtrUnlock(outPtr);
  }

}


// Eventhandler for the form.
Boolean MatchFormEvtHandler(EventPtr event) {
  League  league;
  Boolean handled      = false;
  Boolean updateScreen = false;

  switch (event->eType) {

 // form is opened
  case frmOpenEvent:
    InitMatchForm();
    updateScreen = true;
    handled = true;
    break;

 // form is closed
  case frmCloseEvent:
    FreeMatchForm();
    break;

 // form need to be updated
  case frmUpdateEvent:
    updateScreen = true;
    handled = true;
    break;

 // scrollbar is scrolled
  case sclRepeatEvent:
    updateScreen = true;
    break;

 // one of the scrollbuttons was pressed
  case keyDownEvent:
    switch(event->data.keyDown.chr) {
      case pageUpChr :
        GState.Match.round = max(GState.Match.round-1, 1);
        updateScreen = true;
        handled = true;
        break;
      case pageDownChr:
        league = LeagueGetCurrent();
        GState.Match.round = min(GState.Match.round+1, league.roundCount);
        updateScreen = true;
        handled = true;
        break;
    }
    break;

 // a button was pressed
  case ctlSelectEvent:
    switch(event->data.ctlSelect.controlID) {
      case btnID_table:
        GState.mode = MODE_TABLE;
        FrmGotoForm(formID_table);
        handled = true;
        break;
      case btnID_team:
        GState.mode = MODE_TEAM;
        FrmGotoForm(formID_team);
        handled = true;
        break;
      case btnID_league:
        GState.mode = MODE_LEAGUE;
        FrmGotoForm(formID_league);
        handled = true;
        break;

      case btnID_roundBack:
        GState.Match.round = max(GState.Match.round-1, 1);
        handled = true;
        updateScreen = true;
        break;
      case btnID_roundFwd:
        league = LeagueGetCurrent();
        GState.Match.round = min(GState.Match.round+1, league.roundCount);
        handled = true;
        updateScreen = true;
        break;
    }
    break;

 // drop-down selected
  case popSelectEvent:
    switch(event->data.popSelect.controlID) {
      case popTrigID_league:
        if(event->data.popSelect.selection != GState.currentLeagueIndex) {
          // We need to explicitly save the results before ShowLeague() is
          // called becuase ShowLeague() will first close the db and then
          // close the form (which saves the reults). If the db is closed
          // before the results are saved they will be saved in the wrong db.
          SaveChangedResults();
          ShowLeague(event->data.popSelect.selection, MODE_MATCHES, true);
        }
        break;
    }
    break;

 // the screen was tapped
 // used when the user wants to change the date for a match
  case penDownEvent:
    // tapped on a date
    if((event->screenX >= 0) && (event->screenX <= 22) && (event->screenY >= 39)) {
      Int16 rowIndex = (event->screenY-39) / 11; // integer division
      if(rowIndex < min(GMatchesInRoundCount, VISIBLE_MATCHES)) {
        ChangeDate(rowIndex);
        handled = true;
        updateScreen = true;
      }
    }
    // tapped on the round label
    if(((event->screenX >= 54) && (event->screenX <= 90)) &&
       ((event->screenY >= 26) && (event->screenY <= 36))) {
      FrmPopupForm(formID_selectRound);
      handled = true;
    }
    break;
  }

 // redraw the screen if needed
  if(updateScreen) {
    UpdateMatchForm();
    FrmDrawForm(FrmGetActiveForm());
    DrawMatchForm();
  }

  return handled;
}
