/*
 * TeamStats by Mats Bovin (mats@mbovin.com)
 * v1.3.0 September 2002
 * Copyright (c) 2002 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <PalmOS.h>
#include "TeamStats.h"
#include "rcp.h"
#include "League.h"
#include "Funcs.h"
#include "Table.h"
#include "Text.h"


// Constants
const Int8 TABLE_ROWS   = MAX_TEAMS;
const Int8 VISIBLE_ROWS = 11;

// Global variables
MemHandle GRoundFieldHandle;


// Draws a cell in the table. This is a callback function which is called
// when the table is redrawn.
void DrawTableCell(void* tablePtr, Int16 row, Int16 column, RectanglePtr bounds) {
  Char   buf[15];
  Char*  tempPtr;
  League league = LeagueGetCurrent();
  Int16  x      = bounds->topLeft.x;
  // the data for this cell
  Int16  data   = TblGetItemInt((TablePtr)tablePtr, row, column);

  // what column to draw?
  switch(column) {
    case 0 : // position
      StrIToA(buf, data);
      StrCat(buf, ".");
      // right-align
      x += bounds->extent.x - FntCharsWidth(buf, StrLen(buf)) - 1;
      break;
    case 1 : // team name
      tempPtr = LeagueGetTeam(&league, data);
      StrCopy(buf, tempPtr);
      MemPtrUnlock(tempPtr);
      break;
    case 3 : // goals for
      StrIToA(buf, data);
      StrCat(buf, TEXT_TABLE_GOAL_SEP);
      // right-align
      x += bounds->extent.x - FntCharsWidth(buf, StrLen(buf)) - 1;
      break;
    case 5 : // points
      StrIToA(buf, data);
      // right-align
      x += bounds->extent.x - FntCharsWidth(buf, StrLen(buf)) - 1 - 4;
      break;
    default : // all other columns -> just print the data
      StrIToA(buf, data);
  }

  WinDrawChars(buf, StrLen(buf), x, bounds->topLeft.y);
}


// Initializes the form.
void InitTableForm() {
  Int16        i;
  MemPtr       resPtr;
  ScrollBarPtr tableScl;
  TablePtr     tableTbl;
  League       league = LeagueGetCurrent();

  InitTop(MODE_TABLE);

  UIInitCustomDrawTable(tableID_table, TABLE_ROWS, 6, DrawTableCell);
  // hide rows that wont be needed
  tableTbl = (TablePtr)UIGetObjectPtr(tableID_table);
  for(i = league.teamCount; i < TABLE_ROWS; i++) {
    TblSetRowUsable(tableTbl, i, false);
  }

  tableScl = (ScrollBarPtr)UIGetObjectPtr(scrollID_table);
  SclSetScrollBar(tableScl, GState.Table.scrollPos, 0, max(league.teamCount-VISIBLE_ROWS, 0), VISIBLE_ROWS);

  UISelectDropDown(popListID_matches, popTrigID_matches, GState.Table.matches);
  UISelectDropDown(popListID_rounds, popTrigID_rounds, GState.Table.rounds);

  // init the round count field
  GRoundFieldHandle = MemHandleNew(3);
  resPtr = MemHandleLock(GRoundFieldHandle);
  StrIToA(resPtr, GState.Table.roundCount);
  MemHandleUnlock(GRoundFieldHandle);
  FldSetTextHandle(UIGetObjectPtr(fieldID_roundCount), GRoundFieldHandle);
}


// Frees resources allocated for this form.
void FreeTableForm() {
  FldSetTextHandle(UIGetObjectPtr(fieldID_roundCount), NULL);
  MemHandleFree(GRoundFieldHandle);
}


// Updates the controls and data on the form.
void UpdateTableForm() {
  Int16    i;
  FormPtr  form   = FrmGetActiveForm();
  TablePtr tableTbl;
  Boolean  includeHome = (GState.Table.matches == MATCHES_ALL || GState.Table.matches == MATCHES_HOME);
  Boolean  includeAway = (GState.Table.matches == MATCHES_ALL || GState.Table.matches == MATCHES_AWAY);
  Boolean  forward     = (GState.Table.rounds == 0 || GState.Table.rounds == 1);
  Int16    maxMatches  = (GState.Table.rounds == 0) ? -1 : GState.Table.roundCount;
  League   league = LeagueGetCurrent();
  Table    table  = TableCreate(&league, includeHome, includeAway, forward, maxMatches);

  // show or hide round count field and update button
  if(GState.Table.rounds == 0) {
    FrmHideObject(form, FrmGetObjectIndex(form, fieldID_roundCount));
    FrmHideObject(form, FrmGetObjectIndex(form, btnID_roundUpdate));
  }
  else {
    FrmShowObject(form, FrmGetObjectIndex(form, fieldID_roundCount));
    FrmShowObject(form, FrmGetObjectIndex(form, btnID_roundUpdate));
  }

  // update data for the table
  tableTbl = (TablePtr)UIGetObjectPtr(tableID_table);
  for(i = 0; i < league.teamCount; i++) {
    TableTeam* team = &table.teams[i];
    Int16      pos  = team->position-1;
    TblSetRowUsable(tableTbl, pos, (pos >= GState.Table.scrollPos));
    TblSetItemInt(tableTbl, pos, 0, team->position);
    TblSetItemInt(tableTbl, pos, 1, team->teamId);
    TblSetItemInt(tableTbl, pos, 2, team->played);
    TblSetItemInt(tableTbl, pos, 3, team->gfor);
    TblSetItemInt(tableTbl, pos, 4, team->against);
    TblSetItemInt(tableTbl, pos, 5, team->points);
  }
  // make sure the table is redrawn
  if(FrmVisible(FrmGetActiveForm())) {
    TblEraseTable(tableTbl);
  }
}


// Draws the form.
void DrawTableForm() {
  Int16  i, y;
  League league = LeagueGetCurrent();

  // draw lines in the table
  for (i = 0; i < 4; i++) {
    y = 49 + (league.line[i]-GState.Table.scrollPos)*11;
    if (y >= 49) {
      WinDrawGrayLine(3, y, 152, y);
    }
  }

}


// Scrolls the table.
void ScrollTable(Int8 delta) {
  ScrollBarPtr tableScl;
  League league    = LeagueGetCurrent();
  Int16  teamCount = league.teamCount;

  GState.Table.scrollPos += delta;
  if(GState.Table.scrollPos < 0) {
    GState.Table.scrollPos = 0;
  }
  else if(GState.Table.scrollPos > teamCount-VISIBLE_ROWS) {
    GState.Table.scrollPos = teamCount-VISIBLE_ROWS;
  }
  // update the scrollbar
  tableScl = (ScrollBarPtr)UIGetObjectPtr(scrollID_table);
  SclSetScrollBar(tableScl, GState.Table.scrollPos, 0, max(teamCount-VISIBLE_ROWS, 0), VISIBLE_ROWS);
}


// Eventhandler for the form.
Boolean TableFormEvtHandler(EventPtr event) {
  Boolean handled      = false;
  Boolean updateScreen = false;

  switch (event->eType) {

 // form is opened
  case frmOpenEvent:
    InitTableForm();
    updateScreen = true;
    handled = true;
    break;

 // form is closed
  case frmCloseEvent:
    FreeTableForm();
    break;

 // form need to be updated
  case frmUpdateEvent:
    updateScreen = true;
    handled = true;
    break;

 // scrollbar is scrolled
  case sclRepeatEvent:
    GState.Table.scrollPos  = event->data.sclRepeat.newValue;
    GState.Table.roundCount = UIGetFieldIntValue(fieldID_roundCount);
    updateScreen = true;
    break;

 // one of the scrollbuttons was pressed
  case keyDownEvent:
    switch(event->data.keyDown.chr) {
      case pageUpChr :
        ScrollTable(-VISIBLE_ROWS);
        GState.Table.roundCount = UIGetFieldIntValue(fieldID_roundCount);
        handled = true;
        updateScreen = true;
        break;
      case pageDownChr:
        ScrollTable(VISIBLE_ROWS);
        GState.Table.roundCount = UIGetFieldIntValue(fieldID_roundCount);
        handled = true;
        updateScreen = true;
        break;
    }
    break;

 // a button was tapped
  case ctlSelectEvent:
    switch(event->data.ctlSelect.controlID) {
      case btnID_matches:
        GState.mode = MODE_MATCHES;
        FrmGotoForm(formID_matches);
        handled = true;
        break;
      case btnID_team:
        GState.mode = MODE_TEAM;
        FrmGotoForm(formID_team);
        handled = true;
        break;
      case btnID_league:
        GState.mode = MODE_LEAGUE;
        FrmGotoForm(formID_league);
        handled = true;
        break;

      case btnID_roundUpdate:
        GState.Table.roundCount = UIGetFieldIntValue(fieldID_roundCount);
        handled = true;
        updateScreen = true;
        break;
    }
    break;

 // drop-down selected
  case popSelectEvent:
    switch(event->data.popSelect.controlID) {
      case popTrigID_league:
        if(event->data.popSelect.selection != GState.currentLeagueIndex) {
          ShowLeague(event->data.popSelect.selection, MODE_TABLE, true);
        }
        break;
      case popTrigID_matches:
        GState.Table.matches = event->data.popSelect.selection;
        updateScreen = true;
        break;
      case popTrigID_rounds:
        GState.Table.rounds = event->data.popSelect.selection;
        updateScreen = true;
        break;
    }
    GState.Table.roundCount = UIGetFieldIntValue(fieldID_roundCount);
    break;

  }

 // redraw the screen if needed
  if(updateScreen) {
    UpdateTableForm();
    FrmDrawForm(FrmGetActiveForm());
    DrawTableForm();
  }

  return handled;
}
